# gather gsea results for a database and plot
# ls */*/*/gsea_report*.xls > gsea-report-files.txt

# read path to gsea reports
x = scan('gsea-report-files.txt', 'character')
r = data.frame(file=x, stringsAsFactors=F)
r$samp = sapply(x, function(t) unlist(strsplit(t, "/"))[1])
r$db = sapply(x, function(t) unlist(strsplit(t, "/"))[2])

readParseGSEA <- function(posFile, negFile, sample){
    p = read.table(posFile, header=T, stringsAsFactors=F, sep='\t', quote='',
        comment.char='')
    p = p[, c('NAME', 'NES', 'NOM.p.val', 'FDR.q.val')]
    colnames(p) = c('name', 'nes', 'p', 'fdr')
    n = read.table(negFile, header=T, stringsAsFactors=F, sep='\t', quote='',
        comment.char='')
    n = n[, c('NAME', 'NES', 'NOM.p.val', 'FDR.q.val')]
    colnames(n) = c('name', 'nes', 'p', 'fdr')
    res = rbind(p,n)
    res = res[order(res$p),]; res=res[!duplicated(res$name),]
    #res = res[res$p <= 0.05 & !is.na(res$p),]
    colnames(res) = c('name', paste0(sample, '.', colnames(res)[-1]))
    return(res)
}
r = r[r$db != 'ALL' & grepl('vC', r$samp),]
z = list()
for (db in unique(r$db)){
    cat(db, '\n')
    y = NULL
    for (s in unique(r$samp[r$db == db])){
        files = r$file[r$samp == s & r$db == db]
        if (length(files) != 2){stop('ERR: missing files?\n')}
        x = readParseGSEA(files[1], files[2], s)
        x$db = db
        if (is.null(y)){y = x}else{y = merge(y, x, all=T)}
    }
    fdr.sig = rowSums(y[, grepl('.fdr$', colnames(y))] <= 0.25, na.rm=T) > 0
    p.sig = rowSums(y[, grepl('.p$', colnames(y))] <= 0.05, na.rm=T) > 0
    y = y[p.sig & fdr.sig,]
    z[[db]] = y
}


# plot heat maps
library(gplots)
plotHeatmap <- function(x, label){
    nes.cols = grep('nes$', colnames(x), value=T)
    top.sets = c()
    for (col in nes.cols){
        top.sets = c(top.sets, x$name[order(abs(x[[col]]), decreasing=T)][1:50])
    }
    top.sets = unique(top.sets)
    x = x[x$name %in% top.sets,]
    rownames(x) = x$name
    x = as.matrix(x[, nes.cols])
    colnames(x) = gsub('.nes', '', colnames(x))
    colBreaks = c(-5, seq(-2,2,0.1), 5)
    col = colorpanel(length(colBreaks)-1, low='blue', mid='white', high='red')
    heatmap.2(x, margins=c(10,20), dendrogram='none', Rowv=T, Colv=F, main=label,
        key=T, keysize=1,
        col=col, breaks=colBreaks, trace='none', cexRow=0.5, cexCol=2
        #lhei=c(1,10), lwid=c(2,5)
        )
}

pdf('GSEA.top50.pdf', width=8, height=10)
plotHeatmap(z$H, 'Hallmark')
#plotHeatmap(z$GO, 'Gene ontology')
#plotHeatmap(z$CP[grepl('REACTOME', z$CP$name),], 'Canonical Pathways\n(REACTOME)')
#plotHeatmap(z$CP[grepl('KEGG', z$CP$name),], 'Canonical Pathways\n(KEGG)')
#plotHeatmap(z$CP, 'Canonical Pathways (all)')
#plotHeatmap(z$C2, 'Curated gene sets')
dev.off()

#stop()

# BEGINNNN
db = 'H'
samples = c('H', 'E', 'HE')


posFile.H = r$file[r$db == 'H' & r$samp == 'HvC' & !grepl('CONTROL', r$file)]
posFile.E = r$file[r$db == 'H' & r$samp == 'EvC' & !grepl('CONTROL', r$file)]
posFile.HE = r$file[r$db == 'H' & r$samp == 'HEvC' & !grepl('CONTROL', r$file)]

negFile.H = r$file[r$db == 'H' & r$samp == 'HvC' & grepl('CONTROL', r$file)]
negFile.E = r$file[r$db == 'H' & r$samp == 'EvC' & grepl('CONTROL', r$file)]
negFile.HE = r$file[r$db == 'H' & r$samp == 'HEvC' & grepl('CONTROL', r$file)]

x = readParseGSEA(posFile.E, negFile.E, 'E')
y = x
x = readParseGSEA(posFile.HE, negFile.HE, 'HE')
y = merge(y,x, all=T)
x = readParseGSEA(posFile.H, negFile.H, 'H')
y = merge(y,x, all=T)

x = y
rownames(x) = x$name
x = as.matrix(x[, grep('.nes$', colnames(x))])
colnames(x) = gsub('H.nes', 'Hydrogel', colnames(x))
colnames(x) = gsub('E.nes', 'EGF', colnames(x))
colnames(x) = gsub('HE.nes', 'Hydrogel-EGF', colnames(x))

library(gplots)
colBreaks = c(-5, seq(-3,3,0.1), 5)
col = colorpanel(length(colBreaks)-1, low='blue', mid='white', high='red')
pdf('HALLMARK.pdf', width=8, height=10)
heatmap.2(x, margins=c(20,30), dendrogram='none', Rowv=T, Colv=F,
    col=col, breaks=colBreaks, trace='none', cexRow=0.5, cexCol=0.5
    #lhei=c(1,10), lwid=c(2,5)
    )
dev.off()



